@@ -11,6 +11,7 @@ extern "C" {
 #define R_CHECKS_LEVEL 2
 #define R_CRITICAL_ENABLED 0
 #define DEBUGGER 1
+#define WANT_ZIP 1
 #define HAVE_DECL_ADDR_NO_RANDOMIZE 1
 #ifdef __EMSCRIPTEN__
 #define HAVE_ARC4RANDOM_UNIFORM 0
@@ -22,6 +23,8 @@ extern "C" {
 #define HAVE_CLOCK_NANOSLEEP 1
 #define HAVE_SIGACTION 1
 #define WANT_THREADS 0
+#define WANT_QJS 1
+#define WANT_V35 0
 #define WANT_CAPSTONE 1
 #define HAVE_LINUX_CAN_H 0
 #define R2_USE_NEW_ABI 0
@@ -50,24 +53,26 @@ extern "C" {
 #define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
 #define R2_BINDIR "/usr/bin"
+#define R2_MANDIR "share/man"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/5.9.8/www"
-#define R2_PLUGINS "lib/radare2/5.9.8"
-#define R2_EXTRAS "lib/radare2-extras/5.9.8"
-#define R2_BINDINGS "lib/radare2-bindings/5.9.8"
+#define R2_WWWROOT "/usr/share/radare2/6.0.4/www"
+#define R2_PLUGINS "lib/radare2/6.0.4"
+#define R2_EXTRAS "lib/radare2-extras/6.0.4"
+#define R2_BINDINGS "lib/radare2-bindings/6.0.4"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/5.9.8"
-#define R2_ZIGNS "share/radare2/5.9.8/zigns"
-#define R2_THEMES "share/radare2/5.9.8/cons"
+#define R2_SDB "share/radare2/6.0.4"
+#define R2_ZIGNS "share/radare2/6.0.4/zigns"
+#define R2_THEMES "share/radare2/6.0.4/cons"
 #define R2_FORTUNES R_JOIN_3_PATHS ("/usr/share", "doc", "radare2")
-#define R2_FLAGS "share/radare2/5.9.8/flag"
-#define R2_HUD "share/radare2/5.9.8/hud"
+#define R2_FLAGS "share/radare2/6.0.4/flag"
+#define R2_HUD "share/radare2/6.0.4/hud"
 #else
 #if R2__WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
 #define R2_ETCDIR "etc"
 #define R2_BINDIR "bin"
+#define R2_MANDIR "man"
 #define R2_LIBDIR "lib"
 #define R2_INCDIR "include\\libr"
 #define R2_DATDIR "share"
@@ -80,6 +85,7 @@ extern "C" {
 #define R2_ETCDIR "/usr/etc"
 #define R2_LIBDIR "/usr/lib"
 #define R2_BINDIR "/usr/bin"
+#define R2_MANDIR "share/man"
 #define R2_INCDIR "/usr/include/libr/libr"
 #define R2_DATDIR "/usr/share"
 #define R2_WWWROOT R2_DATDIR "/radare2/" R2_VERSION "/www"
